@@ -22,7 +22,6 @@ my $Re1 = {
     'rfc822' => qr|\AContent-Type: message/rfc822|,
     'endof'  => qr/\A__END_OF_EMAIL_MESSAGE__\z/,
 };
-
 my $CodeTable = {
     'onhold' => [
         '000B099C', # Host Unknown, Message exceeds size limit, ...
@@ -50,10 +49,7 @@ my $CodeTable = {
         '000C0595', # Ambiguous Recipient
     ],
 };
-
 my $Indicators = __PACKAGE__->INDICATORS;
-my $LongFields = Sisimai::RFC5322->LONGFIELDS;
-my $RFC822Head = Sisimai::RFC5322->HEADERFIELDS;
 
 sub description { 'Microsoft Exchange Server' }
 sub smtpagent   { 'Exchange' }
@@ -111,11 +107,11 @@ sub scan {
     }
     return undef unless $match;
 
-    my $dscontents = []; push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
+    my $dscontents = [ __PACKAGE__->DELIVERYSTATUS ];
     my @hasdivided = split( "\n", $$mbody );
-    my $rfc822next = { 'from' => 0, 'to' => 0, 'subject' => 0 };
     my $rfc822part = '';    # (String) message/rfc822-headers part
-    my $previousfn = '';    # (String) Previous field name
+    my $rfc822list = [];    # (Array) Each line in message/rfc822 part string
+    my $blanklines = 0;     # (Integer) The number of blank lines
     my $readcursor = 0;     # (Integer) Points the current cursor position
     my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
     my $statuspart = 0;     # (Integer) Flag, 1 = have got delivery status part.
@@ -148,26 +144,12 @@ sub scan {
 
         if( $readcursor & $Indicators->{'message-rfc822'} ) {
             # After "message/rfc822"
-            if( $e =~ m/\A([-0-9A-Za-z]+?)[:][ ]*.+\z/ ) {
-                # Get required headers only
-                my $lhs = lc $1;
-                $previousfn = '';
-                next unless exists $RFC822Head->{ $lhs };
-
-                $previousfn  = $lhs;
-                $rfc822part .= $e."\n";
-
-            } elsif( $e =~ m/\A[ \t]+/ ) {
-                # Continued line from the previous line
-                $rfc822part .= $e."\n" if exists $LongFields->{ $previousfn };
-                next if $rfc822next->{ $previousfn };
-
-            } else {
-                # Check the end of headers in rfc822 part
-                next unless exists $LongFields->{ $previousfn };
-                next if length $e;
-                $rfc822next->{ $previousfn } = 1;
+            unless( length $e ) {
+                $blanklines++;
+                last if $blanklines > 1;
+                next;
             }
+            push @$rfc822list, $e;
 
         } else {
             # Before "message/rfc822"
@@ -256,12 +238,6 @@ sub scan {
     require Sisimai::SMTP::Status;
 
     for my $e ( @$dscontents ) {
-        if( scalar @{ $mhead->{'received'} } ) {
-            # Get localhost and remote host name from Received header.
-            my $r0 = $mhead->{'received'};
-            $e->{'lhost'} ||= shift @{ Sisimai::RFC5322->received( $r0->[0] ) };
-            $e->{'rhost'} ||= pop @{ Sisimai::RFC5322->received( $r0->[-1] ) };
-        }
         $e->{'diagnosis'} = Sisimai::String->sweep( $e->{'diagnosis'} );
 
         if( $e->{'diagnosis'} =~ m{\AMSEXCH:.+[ \t]*[(]([0-9A-F]{8})[)][ \t]*(.*)\z} ) {
@@ -290,20 +266,18 @@ sub scan {
                 delete $e->{'alterrors'};
             }
         }
-
-        $e->{'spec'}   = $e->{'reason'} eq 'mailererror' ? 'X-UNIX' : 'SMTP';
-        $e->{'action'} = 'failed' if $e->{'status'} =~ m/\A[45]/;
         $e->{'agent'}  = __PACKAGE__->smtpagent;
         delete $e->{'msexch'};
     }
 
-    if( length( $rfc822part ) == 0 ) {
+    if( scalar(@$rfc822list) == 0 ) {
         # When original message does not included in the bounce message
-        $rfc822part .= sprintf( "From: %s\n", $connheader->{'to'} );
-        $rfc822part .= sprintf( "Date: %s\n", $connheader->{'date'} );
-        $rfc822part .= sprintf( "Subject: %s\n", $connheader->{'subject'} );
+        push @$rfc822list, sprintf( "From: %s", $connheader->{'to'} );
+        push @$rfc822list, sprintf( "Date: %s", $connheader->{'date'} );
+        push @$rfc822list, sprintf( "Subject: %s", $connheader->{'subject'} );
     }
-    return { 'ds' => $dscontents, 'rfc822' => $rfc822part };
+    $rfc822part = Sisimai::RFC5322->weedout( $rfc822list );
+    return { 'ds' => $dscontents, 'rfc822' => $$rfc822part };
 }
 
 1;
