@@ -13,7 +13,6 @@ my $Re1 = {
     'rfc822'  => qr/^[-]+[ ]+Returned Message[ ]+[-]+$/,
     'endof'   => qr/\A__END_OF_EMAIL_MESSAGE__\z/,
 };
-
 my $ReFailure = {
     'userunknown' => qr{(?:
          User[ ]not[ ]listed[ ]in[ ]public[ ]Name[ ][&][ ]Address[ ]Book
@@ -22,10 +21,7 @@ my $ReFailure = {
     }x,
     'networkerror' => qr/Message has exceeded maximum hop count/,
 };
-
 my $Indicators = __PACKAGE__->INDICATORS;
-my $LongFields = Sisimai::RFC5322->LONGFIELDS;
-my $RFC822Head = Sisimai::RFC5322->HEADERFIELDS;
 
 sub description { 'Lotus Notes' }
 sub smtpagent   { 'Notes' }
@@ -52,11 +48,11 @@ sub scan {
 
     require Sisimai::Address;
 
-    my $dscontents = []; push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
+    my $dscontents = [ __PACKAGE__->DELIVERYSTATUS ];
     my @hasdivided = split( "\n", $$mbody );
-    my $rfc822next = { 'from' => 0, 'to' => 0, 'subject' => 0 };
     my $rfc822part = '';    # (String) message/rfc822-headers part
-    my $previousfn = '';    # (String) Previous field name
+    my $rfc822list = [];    # (Array) Each line in message/rfc822 part string
+    my $blanklines = 0;     # (Integer) The number of blank lines
     my $readcursor = 0;     # (Integer) Points the current cursor position
     my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
     my $characters = '';    # (String) Character set name of the bounce mail
@@ -92,26 +88,12 @@ sub scan {
 
         if( $readcursor & $Indicators->{'message-rfc822'} ) {
             # After "message/rfc822"
-            if( $e =~ m/\A([-0-9A-Za-z]+?)[:][ ]*.+\z/ ) {
-                # Get required headers only
-                my $lhs = lc $1;
-                $previousfn = '';
-                next unless exists $RFC822Head->{ $lhs };
-
-                $previousfn  = $lhs;
-                $rfc822part .= $e."\n";
-
-            } elsif( $e =~ m/\A[ \t]+/ ) {
-                # Continued line from the previous line
-                next if $rfc822next->{ $previousfn };
-                $rfc822part .= $e."\n" if exists $LongFields->{ $previousfn };
-
-            } else {
-                # Check the end of headers in rfc822 part
-                next unless exists $LongFields->{ $previousfn };
-                next if length $e;
-                $rfc822next->{ $previousfn } = 1;
+            unless( length $e ) {
+                $blanklines++;
+                last if $blanklines > 1;
+                next;
             }
+            push @$rfc822list, $e;
 
         } else {
             # Before "message/rfc822"
@@ -165,23 +147,20 @@ sub scan {
 
     unless( $recipients ) {
         # Fallback: Get the recpient address from RFC822 part
-        if( $rfc822part =~ m/^To:[ ]*(.+)$/m ) {
-            $v->{'recipient'} = Sisimai::Address->s3s4( $1 );
-            $recipients++ if $v->{'recipient'};
+        for my $e ( @$rfc822list ) {
+            if( $e =~ m/^To:[ ]*(.+)$/ ) {
+                $v->{'recipient'} = Sisimai::Address->s3s4( $1 );
+                $recipients++ if $v->{'recipient'};
+                last;
+            }
         }
     }
     return undef unless $recipients;
-
     require Sisimai::String;
     require Sisimai::SMTP::Status;
 
     for my $e ( @$dscontents ) {
-        if( scalar @{ $mhead->{'received'} } ) {
-            # Get localhost and remote host name from Received header.
-            my $r0 = $mhead->{'received'};
-            $e->{'lhost'} ||= shift @{ Sisimai::RFC5322->received( $r0->[0] ) };
-            $e->{'rhost'} ||= pop @{ Sisimai::RFC5322->received( $r0->[-1] ) };
-        }
+        $e->{'agent'}     = __PACKAGE__->smtpagent;
         $e->{'diagnosis'} = Sisimai::String->sweep( $e->{'diagnosis'} );
         $e->{'recipient'} = Sisimai::Address->s3s4( $e->{'recipient'} );
 
@@ -194,12 +173,9 @@ sub scan {
             $e->{'status'} = $pseudostatus if length $pseudostatus;
             last;
         }
-
-        $e->{'spec'}   = $e->{'reason'} eq 'mailererror' ? 'X-UNIX' : 'SMTP';
-        $e->{'action'} = 'failed' if $e->{'status'} =~ m/\A[45]/;
-        $e->{'agent'}  = __PACKAGE__->smtpagent;
     }
-    return { 'ds' => $dscontents, 'rfc822' => $rfc822part };
+    $rfc822part = Sisimai::RFC5322->weedout( $rfc822list );
+    return { 'ds' => $dscontents, 'rfc822' => $$rfc822part };
 }
 
 1;
