@@ -34,10 +34,7 @@ my $Re1 = {
     }x,
     'endof'   => qr/\A__END_OF_EMAIL_MESSAGE__\z/,
 };
-
 my $Indicators = __PACKAGE__->INDICATORS;
-my $LongFields = Sisimai::RFC5322->LONGFIELDS;
-my $RFC822Head = Sisimai::RFC5322->HEADERFIELDS;
 
 sub description { 'Sendmail version 5' }
 sub smtpagent   { 'V5sendmail' }
@@ -62,11 +59,11 @@ sub scan {
 
     return undef unless $mhead->{'subject'} =~ $Re0->{'subject'};
 
-    my $dscontents = []; push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
+    my $dscontents = [ __PACKAGE__->DELIVERYSTATUS ];
     my @hasdivided = split( "\n", $$mbody );
-    my $rfc822next = { 'from' => 0, 'to' => 0, 'subject' => 0 };
     my $rfc822part = '';    # (String) message/rfc822-headers part
-    my $previousfn = '';    # (String) Previous field name
+    my $rfc822list = [];    # (Array) Each line in message/rfc822 part string
+    my $blanklines = 0;     # (Integer) The number of blank lines
     my $readcursor = 0;     # (Integer) Points the current cursor position
     my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
     my @responding = ();    # (Array) Responses from remote server
@@ -95,26 +92,12 @@ sub scan {
 
         if( $readcursor & $Indicators->{'message-rfc822'} ) {
             # After "message/rfc822"
-            if( $e =~ m/\A([-0-9A-Za-z]+?)[:][ ]*.+\z/ ) {
-                # Get required headers only
-                my $lhs = lc $1;
-                $previousfn = '';
-                next unless exists $RFC822Head->{ $lhs };
-
-                $previousfn  = $lhs;
-                $rfc822part .= $e."\n";
-
-            } elsif( $e =~ m/\A[ \t]+/ ) {
-                # Continued line from the previous line
-                next if $rfc822next->{ $previousfn };
-                $rfc822part .= $e."\n" if exists $LongFields->{ $previousfn };
-
-            } else {
-                # Check the end of headers in rfc822 part
-                next unless exists $LongFields->{ $previousfn };
-                next if length $e;
-                $rfc822next->{ $previousfn } = 1;
+            unless( length $e ) {
+                $blanklines++;
+                last if $blanklines > 1;
+                next;
             }
+            push @$rfc822list, $e;
 
         } else {
             # Before "message/rfc822"
@@ -176,10 +159,13 @@ sub scan {
 
     unless( $recipients ) {
         # Get the recipient address from the original message
-        if( $rfc822part =~ m/^To: (.+)$/m ) {
-            # The value of To: header in the original message
-            $dscontents->[0]->{'recipient'} = Sisimai::Address->s3s4( $1 );
-            $recipients = 1;
+        for my $e ( @$rfc822list ) {
+            if( $e =~ m/^To: (.+)$/ ) {
+                # The value of To: header in the original message
+                $dscontents->[0]->{'recipient'} = Sisimai::Address->s3s4( $1 );
+                $recipients = 1;
+                last;
+            }
         }
     }
     return undef unless $recipients;
@@ -187,15 +173,6 @@ sub scan {
 
     for my $e ( @$dscontents ) {
         $errorindex++;
-
-        if( scalar @{ $mhead->{'received'} } ) {
-            # Get localhost and remote host name from Received header.
-            my $r0 = $mhead->{'received'};
-            $e->{'lhost'} ||= shift @{ Sisimai::RFC5322->received( $r0->[0] ) };
-            $e->{'rhost'} ||= pop @{ Sisimai::RFC5322->received( $r0->[-1] ) };
-        }
-
-        $e->{'spec'}  ||= 'SMTP';
         $e->{'agent'}   = __PACKAGE__->smtpagent;
         $e->{'command'} = $commandset[ $errorindex ] || '';
 
@@ -218,7 +195,8 @@ sub scan {
         }
         delete $e->{'sessionerr'};
     }
-    return { 'ds' => $dscontents, 'rfc822' => $rfc822part };
+    $rfc822part = Sisimai::RFC5322->weedout( $rfc822list );
+    return { 'ds' => $dscontents, 'rfc822' => $$rfc822part };
 }
 
 1;
