@@ -70,46 +70,14 @@ my $AnotherList9 = [
     'Sisimai::MSP::DE::GMX',
     'Sisimai::MSP::RU::Yandex',
     'Sisimai::MSP::US::ReceivingSES',
+    'Sisimai::MSP::US::Office365',
+    'Sisimai::MSP::US::AmazonWorkMail',
     'Sisimai::MSP::US::Zoho',
     'Sisimai::MTA::McAfee',
     'Sisimai::MTA::Activehunter',
     'Sisimai::MTA::SurfControl',
 ];
 
-# my $SpecificHead = {
-#     'x-yandex-uniq'         => { 'Sisimai::MSP::RU::Yandex' => 1 },
-#     'x-ymailisg'            => { 'Sisimai::MSP::US::Yahoo' => 1 },
-#     'x-message-info'        => { 'Sisimai::MSP::US::Outlook' => 1 },
-#     'x-ahmailid'            => { 'Sisimai::MTA::Activehunter' => 1 },
-#     'x-nai-header'          => { 'Sisimai::MTA::McAfee' => 1 },
-#     'x-sef-processed'       => { 'Sisimai::MTA::SurfControl' => 1 },
-#     'x-mxl-hash'            => { 'Sisimai::MTA::MXLogic' => 1 },
-#     'x-ms-embedded-report'  => { 'Sisimai::MTA::Exchange' => 1 },
-#     'x-ses-outgoing'        => { 'Sisimai::MSP::US::ReceivingSES' => 1 },
-#     'x-msg-ref'             => { 'Sisimai::MSP::UK::MessageLabs' => 1 },
-#     'x-message-delivery'    => { 'Sisimai::MSP::US::Outlook' => 1 },
-#     'x-gmx-antispam'        => { 'Sisimai::MSP::DE::GMX' => 1 },
-#     'x-mimeole'             => { 'Sisimai::MTA::Exchange' => 1 },
-#     'x-spasign'             => { 'Sisimai::MSP::JP::EZweb' => 1 },
-#     'x-zohomail'            => { 'Sisimai::MSP::US::Zoho' => 1 },
-#     'x-aol-ip'              => { 'Sisimai::MSP::US::Aol' => 1 },
-#     'x-aws-outgoing'        => { 'Sisimai::MSP::US::AmazonSES' => 1 },
-#     'x-mxl-notehash'        => { 'Sisimai::MTA::MXLogic' => 1 },
-#     'x-originating-ip'      => { 'Sisimai::MSP::UK::MessageLabs' => 1 },
-#     'x-failed-recipients'   => {
-#         'Sisimai::MSP::RU::MailRu' => 1,
-#         'Sisimai::MTA::Exim' => 1,
-#         'Sisimai::MSP::US::Google' => 1
-#     },
-#     'x-mailer' => {
-#         'Sisimai::MSP::US::SendGrid' => 1,
-#         'Sisimai::MTA::mFILTER' => 1,
-#         'Sisimai::MTA::Exchange' => 1,
-#         'Sisimai::MTA::IMailServer' => 1,
-#         'Sisimai::MSP::US::Zoho' => 1,
-#     },
-# };
-
 # This variable don't hold MTA/MSP name which have one or more MTA specific
 # header such as X-AWS-Outgoing, X-Yandex-Uniq.
 my $PatternTable = {
@@ -154,6 +122,7 @@ my $PatternTable = {
             'Sisimai::MTA::Postfix',
             'Sisimai::MTA::Exchange',
             'Sisimai::MTA::Notes',
+            'Sisimai::MSP::US::Office365',
             'Sisimai::MSP::US::Verizon',
             'Sisimai::MSP::US::SendGrid',
             'Sisimai::MTA::IMailServer',
